﻿using System;
using System.Collections.ObjectModel;
using System.Diagnostics;
using System.Drawing;
using System.Drawing.Imaging;
using System.IO;
using System.Web.UI;
using PdfSharp;
using PdfSharp.Pdf;
using ThinkGeo.MapSuite.Core;
using ThinkGeo.MapSuite.WebEdition;

namespace DisplayASimpleMap
{
    public partial class _Default : System.Web.UI.Page
    {
        protected void Page_Load(object sender, EventArgs e)
        {
            if (!Page.IsPostBack)
            {
                LayerOverlay staticOverlay = new LayerOverlay();
                Map1.MapBackground.BackgroundBrush = new GeoSolidBrush(GeoColor.FromHtml("#E5E3DF"));
                Map1.CurrentExtent = new RectangleShape(-166.1982421875, 84.615625, -66.7939453125, 11.66640625);
                Map1.MapUnit = GeographyUnit.DecimalDegree;

                ShapeFileFeatureLayer worldLayer = new ShapeFileFeatureLayer(MapPath("~/Data/USStates.shp"));
                worldLayer.ZoomLevelSet.ZoomLevel01.ApplyUntilZoomLevel = ApplyUntilZoomLevel.Level20;
                worldLayer.ZoomLevelSet.ZoomLevel01.DefaultAreaStyle = AreaStyles.Country1;

                ScaleBarAdornmentLayer scaleBar = new ScaleBarAdornmentLayer();
                scaleBar.Location = AdornmentLocation.UpperRight;
                staticOverlay.IsBaseOverlay = false;
                staticOverlay.Layers.Add(worldLayer);
                Map1.AdornmentOverlay.Layers.Add(scaleBar);
                Map1.CustomOverlays.Add(staticOverlay);
            }
        }

        protected void Button_Click(object sender, EventArgs e)
        {
            using (PdfDocument pdfDocument = new PdfDocument())
            {
                PdfPage pdfPage = pdfDocument.AddPage();
                pdfPage.Orientation = PageOrientation.Landscape;
                pdfPage.Size = PageSize.A2;

                PdfGeoCanvas pdfGeoCanvas;
                Stream stream = GetStream();
                pdfGeoCanvas = new PdfGeoCanvas();
                pdfGeoCanvas.BeginDrawing(pdfPage, Map1.CurrentExtent, Map1.MapUnit);
                pdfGeoCanvas.DrawScreenImage(new GeoImage(stream), (float)Map1.WidthInPixels / 2, (float)Map1.HeightInPixels / 2, (float)Map1.WidthInPixels, (float)Map1.HeightInPixels, DrawingLevel.LevelOne, 0, 0, 0);
                pdfGeoCanvas.EndDrawing();
                string fileName = "printing.pdf";
                pdfDocument.Save(fileName);
                Process.Start(fileName);
            }
        }

        private Stream GetStream()
        {
            using (Bitmap bitmap = new Bitmap((int)Map1.WidthInPixels, (int)Map1.HeightInPixels))
            {
                MemoryStream stream = new MemoryStream();
                GdiPlusGeoCanvas geoCanvas = new GdiPlusGeoCanvas();
                geoCanvas.BeginDrawing(bitmap, Map1.CurrentExtent, GeographyUnit.DecimalDegree);
                Collection<SimpleCandidate> labelsInAllLayers = new Collection<SimpleCandidate>();
                foreach (var printerLayer in ((LayerOverlay)Map1.CustomOverlays[0]).Layers)
                {
                    printerLayer.Open();
                    printerLayer.Draw(geoCanvas, labelsInAllLayers);
                }
                foreach (var printerLayer in Map1.AdornmentOverlay.Layers)
                {
                    printerLayer.Open();
                    printerLayer.Draw(geoCanvas, labelsInAllLayers);
                }
                geoCanvas.EndDrawing();

                stream = new MemoryStream();
                bitmap.Save(stream, ImageFormat.Png);
                return stream;
            }
        }
    }
}
